/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.binding;

import org.eclipse.stp.sca.BaseReference;
import org.eclipse.stp.sca.BaseService;
import org.eclipse.stp.sca.Binding;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotations.Interface;

@Interface(name = "binding-protocol")
public interface ScaBindingProtocol {
  
  /** plug-in id constant for binding factory */
  public static String PLUGIN_ID = "plugin.id";
  
  /** classloader constant for binding factory */
  public static String CLASSLOADER = "classloader";
  
  /**
   * Get binding type 
   * 
   * @return EObject ID from the SCA model
   */
  
  public int getBindingID();
  
  /**
   * Bind a Reference to its target
   * 
   * @param reference - the reference interface to bind
   * @param binding - the binding for this reference
   * @param component - the component instance to bind
   */
  
  public void bind(BaseReference reference, Binding binding, Component component);
  
  /**
   * Export a service
   * 
   * @param service - the service interface to export
   * @param binding - the binding for this service
   * @param component - the component instance to export
   */
  
  public void export(BaseService service, Binding binding, Component component);

}
