/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 *
 * Contributor: Philippe Merle
 */

package org.ow2.frascati.factory.core.instance.implementation;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.stp.sca.Implementation;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.fraclet.annotations.Component;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.types.Cardinality;
import org.objectweb.fractal.fraclet.types.Policy;
import org.objectweb.fractal.fraclet.types.Step;
import org.ow2.frascati.factory.core.instance.NotSupportedException;

/**
 * Implementation switcher component.
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

@Component
public class ScaImplementationSwitch implements ScaImplementation {
  // ======================================================================
  //
  // Internal state.
  //
  // ======================================================================

  /**
   * Logger for this class.
   */
  // TODO: This must be put into a class inherited by all components requiring a
  // logger.
  private java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(ScaImplementationSwitch.class.getCanonicalName());

  /**
   * Map of the Implementation plugins indexed by name.
   */
  @Requires(name = "implementations", cardinality = Cardinality.COLLECTION)
  private Map<String, ScaImplementation> implementationPlugins = new HashMap<String, ScaImplementation>();

  /**
   * Map of the Implementation plugins indexed by ImplementationID.
   */
  private Map<Integer, ScaImplementation> implementationIDMap;

  // ======================================================================
  //
  // Constructor.
  //
  // ======================================================================

  // ======================================================================
  //
  // Internal methods.
  //
  // ======================================================================

  /**
   * Implementation Switch initializer, build map for accessing implementation
   * plugins according to the Implementation ID.
   */
  @Lifecycle(step=Step.START, policy=Policy.ASYNCHRONOUS)
  protected void initialize() {

    // Construct map for accessing implementation plugins using EMF Classifier
    // ID.
    this.implementationIDMap = new HashMap<Integer, ScaImplementation>();
    for (String key : implementationPlugins.keySet()) {
      ScaImplementation implementation = implementationPlugins.get(key);
      implementationIDMap.put(implementation.getImplementationID(),
          implementation);
    }
  }

  // ======================================================================
  //
  // Implementation of
  // org.ow2.frascati.factory.core.instance.implementation.ScaImplementation
  //
  // ======================================================================

  /*
   * (non-Javadoc)
   * 
   * @see ScaImplementation#create(Implementation,ComponentType)
   */
  public org.objectweb.fractal.api.Component create(
      Implementation implementation, ComponentType componentType) throws ImplementationException, NotSupportedException {

    // get the implementation plugin according to the EMF Classifier ID.
    ScaImplementation impl = this.implementationIDMap.get(implementation
        .eClass().getClassifierID());

    // If no implementation plugin then throw an exception.
    if (impl == null) {
      throw new NotSupportedException("Implementation type not supported : " + implementation.eClass().getName());
    }

    return impl.create(implementation, componentType);
  }

  /*
   * (non-Javadoc)
   * 
   * @see ScaImplementation#getImplementationID()
   */
  public int getImplementationID() {
    return org.eclipse.stp.sca.ScaPackage.IMPLEMENTATION;
  }
}
