/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.instance.intent;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.emf.ecore.EObject;
import org.objectweb.fractal.fraclet.annotations.Lifecycle;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.fractal.fraclet.types.Cardinality;
import org.objectweb.fractal.fraclet.types.Policy;
import org.objectweb.fractal.fraclet.types.Step;

@org.objectweb.fractal.fraclet.annotations.Component
public class ScaIntentSwitch implements ScaIntent {

  @Requires(name = "intents", cardinality = Cardinality.COLLECTION)
  private HashMap<String, ScaIntent> intentPlugins = new HashMap<String, ScaIntent>();

  /**
   * Map for intent plugins indexed by EObjectID.
   */

  private Map<Integer, ScaIntent> intentIDMap;

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.intent.ScaIntent#setIntent(org.objectweb
   * .fractal.api.Component, java.lang.String)
   */

  // Delegates setIntent() on the component responsible for managing Intents for
  // the given EObject
  public void setIntent(Object o, EObject modelElement) throws IntentException {
    intentIDMap.get(modelElement.eClass().getClassifierID()).setIntent(o,
        modelElement);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.ow2.frascati.factory.core.instance.intent.ScaIntent#getIntentType()
   */
  public int getEObjectID() {
    return -1;
  }

  /**
   * Switch initializer, build map for accessing intent plugin according to the
   * given EObject.
   */
  
  @Lifecycle(step = Step.START, policy = Policy.ASYNCHRONOUS)
  protected void initialize() {

    // Construct map for accessing intent plugins using EMF Classifier
    // ID.
    this.intentIDMap = new HashMap<Integer, ScaIntent>();
    for (String key : intentPlugins.keySet()) {
      ScaIntent intent = intentPlugins.get(key);
      intentIDMap.put(intent.getEObjectID(), intent);
    }
  }

}
