/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.core.parser.util;

import java.io.IOException;

import org.eclipse.emf.common.util.EList;
import org.eclipse.stp.sca.Composite;
import org.ow2.frascati.factory.core.instance.runtime.RuntimeFactory;
import org.ow2.frascati.factory.core.parser.Sca2Ecore;

/**
 * Add to current Composite the components, services, references, properties and
 * wires defined in included composite
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 */

public class IncludeResolver implements org.eclipse.stp.sca.introspection.Resolver {

  /**
   * parser instance used for recursive calls
   */

  protected Sca2Ecore parser;

  /**
   * Reference on runtime component
   */

  protected RuntimeFactory runtime;

  /**
   * Initialize the include Resolver with a parser instance
   * 
   * @param parser
   */

  public IncludeResolver(Sca2Ecore parser, RuntimeFactory runtime) {
    this.parser = parser;
    this.runtime = runtime;
  }

  /**
   * Logger for this class
   */

  java.util.logging.Logger log = java.util.logging.Logger
      .getLogger(JavaInterfaceResolver.class.getCanonicalName());

  /**
   * Resolve <include> for the provided Composite model element
   * 
   * @param composite
   * @return - a Composite with <include> resolved
   * @throws IOException
   */

  public Composite resolve(Composite composite) {

    EList<String> list = composite.getInclude();

    if (list.isEmpty()) {
      // Include not defined in main composite file : nothing to do
      return composite;
    } else {
      // Include present
      for (String include : list) {
        try {
          composite = resolveIncluded(composite, parser.parse(runtime
              .getCompositeResource(include)));
        } catch (IOException e) {
          log.severe("Included File " + include + "could not be loaded");
        }
      }

    }

    return composite;

  }

  /**
   * Internal recursive call
   * 
   * @param composite
   *          - current composite
   * @param includedComposite
   *          - composite found in the <include> element
   * @return - Merged composite
   * @throws IOException
   */

  private Composite resolveIncluded(Composite composite,
      Composite includedComposite) throws IOException {

    EList<String> list = includedComposite.getInclude();

    if (!list.isEmpty()) {
      for (String include : list) {

        includedComposite = resolveIncluded(includedComposite, parser
            .parse(runtime.getCompositeResource(include)));

      }
    }

    // Include is actually not supported with PolicySets
    // composite.getPolicySets().addAll(includedComposite.getPolicySets());

    composite.getComponent().addAll(includedComposite.getComponent());
    composite.getProperty().addAll(includedComposite.getProperty());
    composite.getService().addAll(includedComposite.getService());
    composite.getReference().addAll(includedComposite.getReference());
    composite.getWire().addAll(includedComposite.getWire());

    return composite;

  }

}
