/***
 * OW2 FraSCAti explorer
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 *
 * Contributor(s):
 * 
 */

package org.ow2.frascati.explorer;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.util.StringTokenizer;

import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;

import org.objectweb.fractal.fraclet.annotations.Attribute;
import org.objectweb.fractal.fraclet.annotations.Interface;
import org.objectweb.fractal.fraclet.annotations.Requires;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.api.Tree;
import org.objectweb.util.explorer.parser.api.ParserConfiguration;
import org.objectweb.util.explorer.swing.api.Explorer;
import org.objectweb.util.explorer.swing.api.StatusBar;
import org.objectweb.util.explorer.swing.api.ViewPanel;
import org.objectweb.util.explorer.swing.lib.DefaultTreePanel;

/**
 * The main FraSCAti Framework Explorer GUI.
 *
 * @author Christophe Demarey
 */
@org.objectweb.fractal.fraclet.annotations.Component(provides=@Interface(name="r", signature=Runnable.class))
public class ExplorerGUI
  implements Runnable
{
  // --------------------------------------------------------------------------
  // Internal state
  // --------------------------------------------------------------------------

  /**
   * To store the ExplorerGUI singleton used by some static methods.
   */
  protected static ExplorerGUI singleton;

  /**
   * The title of the frame.
   */
  @Attribute
  protected String frameTitle;

  /**
   * The explorer configuration files to use.
   */
  @Attribute
  protected String explorerConfigurationFiles;

  /**
   * The role used at the beginning.
   */
  @Attribute
  protected String initialRole;

  /**
   * The configuration parser.
   */
  @Requires(name="parser-configuration")
  protected ParserConfiguration parser ;
  
  /**
   * The Explorer engine.
   */
  @Requires(name="explorer")
  protected Explorer explorer;
  
  /**
   * The Explorer Tree.
   */
  @Requires(name="tree")
  protected Tree explorerTree;
  
  /**
   * The Explorer view panel.
   */
  @Requires(name="view-panel")
  protected ViewPanel viewPanel;
  
  /**
   * The Explorer Status bar.
   */
  @Requires(name="status-bar")
  protected StatusBar statusBar;
  
  /**
   * The Explorer SCA Domain Manager.
   */
  @Requires(name="scadomain-manager")
  protected DomainManager domainManager;
  
  /**
   * The Explorer GUI frame.
   */
  protected JFrame frame;

  // --------------------------------------------------------------------------
  // Internal methods
  // --------------------------------------------------------------------------

  // --------------------------------------------------------------------------
  // Constructor
  // --------------------------------------------------------------------------

  /**
   * The default constructor.
   */
  public ExplorerGUI () { 
    ExplorerGUI.singleton = this;
  }

  // --------------------------------------------------------------------------
  // Implementation of the Runnable interface
  // --------------------------------------------------------------------------

  /**
   * Run the GUI.
   */
  public void run () {
    //
    // Configures the explorer engine.
    //

    if (explorerConfigurationFiles != null) {
      StringTokenizer st = new StringTokenizer(explorerConfigurationFiles, "; ");
      
      while (st.hasMoreTokens()) {
        String theFile = st.nextToken();
        parser.addPropertyFile(theFile);
      }
      parser.parse();
    }

    explorer.setCurrentRoles(new String[] { initialRole });

    //
    // Creates the GUI.
    //

    frame = new JFrame(frameTitle);

    JMenuBar menuBar = new JMenuBar();
    frame.setJMenuBar(menuBar);
    explorer.setMenuBar(menuBar);

    JPanel panel = new JPanel(new BorderLayout());
    frame.getContentPane().add(panel);

    JToolBar toolBar = new JToolBar();
    panel.add(toolBar,BorderLayout.NORTH);
    explorer.setToolBar(toolBar);

    panel.add(new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, true,
                             new DefaultTreePanel(explorer.getTree()),
                             viewPanel.getViewPanel() ),
              BorderLayout.CENTER);
    
    panel.add(statusBar.getStatusBar(), BorderLayout.SOUTH);

    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.pack();

    // Centers the frame.
    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
    frame.setLocation((screenSize.width - frame.getWidth()) / 2,
                      (screenSize.height - frame.getHeight()) / 2);

    frame.setVisible(true);
    
    // Add the root component
    for ( Entry entry : domainManager.getEntries(null) )
      explorerTree.addEntry(entry.getName(), entry.getValue(), 1);
    
    // Init the domain manager
    domainManager.init();
  }

  // --------------------------------------------------------------------------
  // Public methods.
  // --------------------------------------------------------------------------

  /**
   * Get the ExplorerGUI singleton.
   */
  public static ExplorerGUI getSingleton() {
      return singleton;
  }
  
}
