/***
 * OW2 FraSCAti explorer
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Christophe Demarey
 *
 * Contributor(s):
 * 
 */

package org.ow2.frascati.explorer.scaproperties;

import javax.swing.JTable;

import org.ow2.frascati.tinfi.control.property.SCAPropertyController;

/**
 * A <code>Thread</code> responsible for periodically testing the value of the
 * monitored property.
 * 
 * @author Christophe Demarey.
 * @author Jakub Kornas 
 * @author Philippe Merle 
 */
public class PropertyMonitoringThread extends Thread
{
  int                   monitoringPeriod;
  SCAPropertyController monitoredObject;
  String                monitoredProperty;
  JTable                table;
  int                   row;
  /** <code>running</code> . */
  public boolean running = false;

  /**
   * Creates a new property monitoring thread.
   * 
   * @param monitoringPeriod the period of testing the property's value.
   * @param propertyController the object providing access to the monitored
   *          property's value.
   * @param monitoredProperty the name of the property to be monitored.
   * @param table a <code>JTable</code> that should be updated when the
   *          monitored property's value changes.
   * @param row a row of the table which stores the property monitored by this
   *          thread.
   */
  public PropertyMonitoringThread(int monitoringPeriod,
                                  SCAPropertyController propertyController, 
                                  String monitoredProperty, 
                                  JTable table, int row)
  {
    this.monitoringPeriod = monitoringPeriod;
    this.monitoredProperty = monitoredProperty;
    this.table = table;
    this.monitoredObject = propertyController;
    this.row = row;
  }

  /**
   * Get the up to date value of the monitored property.
   */
  public void run()
  {
    if (monitoredObject != null) {
        Object newValue = null;
    
        running = true;
        while (running)
        {
          try {
            newValue = monitoredObject.get(monitoredProperty);
            table.setValueAt(newValue, row, 1);
          } catch (Exception e) {
            e.printStackTrace();
          }
    
          try {
            Thread.sleep(monitoringPeriod);
          } catch (InterruptedException e) {
            e.printStackTrace();
          }
        }
    }
  }
}
