/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author(s): Damien Fournier
 *            Christophe Demarey
 */

package org.ow2.frascati.factory;

import java.io.File;
import java.io.IOException;

/**
 * The GenerateFactory generates stuff for a SCA composite description.
 * 
 * @author <a href="mailto:Damien.Fournier@inria.fr">Damien Fournier</a>
 * @author <a href="mailto:Lionel.Seinturier@lifl.fr">Lionel Seinturier</a>
 * @author Christophe Demarey
 */
public abstract class AbstractGenerateFactory extends AbstractFactory {

  /** true to obtain debug information about the code which is being generated. */
  protected boolean debug = false;

  /** default output directory for generated classes */
  protected File output = new File("");

  // Getters and Setters

  public void setDebug(boolean debug) {
    this.debug = debug;
  }
  
  /**
   * Set output directory for generated files
   * 
   * @param output - output directory
   */
  
  public abstract void setOutput(String output);
  
  /**
   * Get output directory for generated files
   * 
   * @return output directory
   * @throws IOException
   */
  
  public String getOutput() throws IOException {
    return output.getCanonicalPath();
  }

}