/***
 * OW2 FraSCAti Assembly Factory
 * Copyright (C) 2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org 
 *
 * Author: Damien Fournier
 */

package org.ow2.frascati.factory.runtime.domain;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.factory.Factory;
import org.objectweb.fractal.util.Fractal;
import org.ow2.frascati.factory.runtime.domain.api.Domain;
import org.ow2.frascati.factory.runtime.domain.api.DomainConfig;

/**
 * Create FraSCAti Domain component instance from java and provides
 * accessors to the Domain and DomainConfig interface
 * 
 * @author <a href="mailto:damien.fournier@inria.fr">Damien Fournier</a>
 * @version $Revision$
 * @since 0.6
 */

public class SCADomain {

  public static String DOMAIN_DEFINITION = "org.ow2.frascati.factory.FrascatiDomain";
  public static String DOMAIN_ITF = "domain";
  public static String DOMAIN_CONF_ITF = "domain-config";

  private Component scaDomain;

  /**
   * Create a new FraScati Domain component 
   */
  
  public SCADomain() {

    try {
      Class<?> cl = Class.forName(DOMAIN_DEFINITION);
      Object o = cl.newInstance();
      scaDomain = ((Factory) o).newFcInstance();
      Fractal.getLifeCycleController(scaDomain).startFc();
    } catch (Exception e) {
      e.printStackTrace();
    }

  }

  /**
   * Get Domain interface from the FraSCAti domain component
   * 
   * @return - reference on Domain interface
   */
  
  public Domain getDomain() {
    try {
      return (Domain) scaDomain.getFcInterface(DOMAIN_ITF);
    } catch (NoSuchInterfaceException e) {
      e.printStackTrace();
    }
    return null;
  }
  
  /**
   * Get DomainConfig interface from the FraSCAti domain component
   * 
   * @return - reference on DomainConfig interface
   */
  
  public DomainConfig getDomainConfig() {
    try {
      return (DomainConfig) scaDomain.getFcInterface(DOMAIN_CONF_ITF);
    } catch (NoSuchInterfaceException e) {
      e.printStackTrace();
    }
    return null;
  }

}
