/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.juliac;

import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.juliac.Utils;
import org.objectweb.fractal.juliac.proxy.ProxyClassGenerator;
import org.objectweb.fractal.juliac.visit.MethodSourceCodeVisitor;

/**
 * This class generates the source code for initializing Fractal components.
 * 
 * This initializer class generator assumes that the content, the interceptors
 * and the controllers are kept in separate classes and that the controllers are
 * implemented with components.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class InitializerCompCtrlSourceCodeGenerator
extends org.objectweb.fractal.juliac.opt.comp.InitializerCompCtrlClassGenerator {

    @Override
    protected void generateNewFcInstanceMethod( MethodSourceCodeVisitor mv ) {
        
        String ctrlDesc = membraneDesc.getDescriptor();
        String contentClassName = Utils.getContentClassName(ctrlDesc,contentDesc);

        mv.visit("    Object content = ");
        if( contentClassName == null || 
            ! ( ctrlDesc.equals("mPrimitive") || ctrlDesc.equals("mComposite") ) ) {
            mv.visitln("null;");
        }
        else {
            mv.visitln("new "+contentClassName+"();");
        }
        
        mv.visitln("    return newFcInstance(content);");
    }

    @Override
    protected ProxyClassGenerator getInterfaceSourceCodeGenerator( 
        InterfaceType it ) {
            
        String ctrlDesc = membraneDesc.getDescriptor();
        InterfaceSourceCodeGeneratorFactory iscgf =
            new InterfaceSourceCodeGeneratorFactory(it,ctrlDesc,membraneDesc,jc);
        ProxyClassGenerator scg = iscgf.getInterfaceSourceCodeGenerator();
        return scg;
    }

    @Override
    protected void generateNewFcContentMethod( MethodSourceCodeVisitor mv ) {
        mv.visitln("    return null;");
    }

    @Override
    protected void generateContentChecks( MethodSourceCodeVisitor mv ) {
        // Indeed nothing
    }

    @Override
    protected void generateInitializationContextForContent( MethodSourceCodeVisitor mv ) {
        mv.visit  ("    ic.content = ");
        mv.visit  (Utils.javaifyContentDesc(contentDesc).toString());
        mv.visitln(";");
    }
}
