/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.Type;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.julia.BasicComponentInterface;
import org.ow2.frascati.tinfi.control.content.SCAContentController;

/**
 * The root class for Tinfi component callback interfaces.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public class TinfiComponentCBInterface extends BasicComponentInterface {

    private SCAContentController cc;

    public TinfiComponentCBInterface() { super(); }
    public TinfiComponentCBInterface( Component owner, String s, Type type, boolean flag, Object obj ) {
        super(owner,s,type,flag,obj);
    }

    /**
     * Query the content controller to retrieve the content instance
     * corresponding to the current scope policy.
     */
    public Object getFcItfImpl() {
        SCAContentController cc = getCC();
        try {
            Object content = cc.getFcContent();
            return content;
        }
        catch( InstantiationException e ) {
            throw new TinfiRuntimeException(e);
        }
    }
    
    /**
     * Method inherited from {@link ComponentInterface} but meaningless in this
     * case. There is not a unique content instance per component. So there is
     * no such thing as setting the reference to the delegate instance.
     */
    public void setFcItfImpl(Object obj) {}
    
    /**
     * Return the reference of the SCA content controller associated to the
     * current component.
     */
    protected SCAContentController getCC() {
        try {
            if( cc == null ) {
                cc = (SCAContentController)
                    owner.getFcInterface(SCAContentController.NAME);
            }
            return cc;
          }
          catch( NoSuchInterfaceException e ) {
              throw new TinfiRuntimeException(e);
          }
    }
}
