/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2007-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.control.content;

import java.util.Stack;

import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.julia.type.BasicInterfaceType;
import org.osoa.sca.RequestContext;

/**
 * Content control interface for SCA primitive components.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 */
public interface SCAContentController {
    
    /** <code>NAME</code> of the content controller. */
    final public static String NAME = "/sca-content-controller";
    
    /** <code>TYPE</code> of the content controller. */
    final public static InterfaceType TYPE =
        new BasicInterfaceType(
            NAME,
            SCAContentController.class.getName(),
            false, false, false );    

    /**
     * Declares the name of the content class which should be used. This
     * operation can only be performed if the component is stopped.
     * 
     * @param name  the content class name
     * @throws IllegalLifeCycleException  if the component is not stopped
     * @throws InstantiationException  if the content can not be instantiated
     * @since 0.6
     */
    public void setFcContentClassName( String name )
    throws IllegalLifeCycleException, InstantiationException;
    
    /**
     * Return a content instance according to the scope policy defined for this
     * component.
     * 
     * @throws InstantiationException  if the content can not be instantiated
     */
    public Object getFcContent() throws InstantiationException;
    
    /**
     * Notify the content controller that the specified content instance is no
     * longer needed. If relevant with the the scope policy, this gives the
     * controller the opportunity to call the {@link Destroy} annotated
     * method on the content instance.
     * 
     * @param content  the content instance which is released
     * @param isEndMethod
     *      <code>true</code> if the method which releases the content instance
     *      is annotated with {@link org.osoa.sca.annotations.EndsConversation}
     */
    public void releaseFcContent( Object content, boolean isEndMethod );

    /**
     * Return the thread-local stack of request contexts.
     */
    public Stack<RequestContext> getRequestContextStack();
    
    /**
     * Set the thread-local stack of request contexts.
     */
    public void setRequestContextStack( Stack<RequestContext> stack );

    /**
     * Eager initialize the content instance associated with this component.
     * Relevant only for composite-scoped components.
     */
    public void eagerInit() throws InstantiationException;
}
