/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2008-2009 INRIA, USTL
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Authors: Lionel Seinturier, Philippe Merle
 */

package org.ow2.frascati.tinfi.control.content.scope;

import java.util.Stack;

import org.objectweb.fractal.api.factory.InstantiationException;
import org.osoa.sca.RequestContext;

/**
 * Interface for scope managers.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @author Philippe Merle <Philippe.Merle@inria.fr>
 */
public interface ScopeManager {

    /**
     * Return a content instance according to the current scope policy.
     * 
     * @throws InstantiationException  if the content can not be instantiated
     */
    public Object getFcContent() throws InstantiationException;
    
    /**
     * Release the specified content instance.
     * 
     * @param content  the content instance which is released
     * @param isEndMethod
     *      <code>true</code> if the method which releases the content instance
     *      is annotated with {@link org.osoa.sca.annotations.EndsConversation}.
     */
    public void releaseFcContent( Object content, boolean isEndMethod );

    /**
     * Return the thread-local stack of request contexts.
     */
    public Stack<RequestContext> getRequestContextStack();
    
    /**
     * Set the thread-local stack of request contexts.
     */
    public void setRequestContextStack( Stack<RequestContext> stack );
}
